import calendar
import os
from django.contrib.auth.models import User
from django.db.models import Count
from django.http import HttpResponse, Http404
from django.shortcuts import render

# Create your views here.
import json

from django.views.decorators.csrf import csrf_exempt

from catalog.models import Search, Purchase

dirname = os.path.dirname(__file__)
configfile = os.path.join(dirname, '../config/config.json')

with open(configfile, 'r') as f:
    config = json.load(f)

USERS_PATH = config['PATHS']['PATH_NAS']
PATH_USERS = config['PATHS']['PATH_USERS']
size_sentinel = 0
size_repsat = 0


# -------------------------------------------
def Users(request):
    user_data = []
    user_searches = []
    user = User.objects.get(pk=request.user.id)

    searches = Search.objects.filter(user_id=user.id)

    purchases = Purchase.objects.filter(user_id=user.id)
    in_process = Purchase.objects.filter(user_id=user.id).exclude(progress=100)

    bytes_size = 0
    for path, directories, files in os.walk(USERS_PATH + "repsat_test_dev/" + user.email):
        for filename in files:
            bytes_size += os.lstat(os.path.join(path, filename)).st_size
        for directory in directories:
            bytes_size += os.lstat(os.path.join(path, directory)).st_size
    bytes_size += os.path.getsize(USERS_PATH + "repsat_test_dev/" + user.email)
    size_repsat = best_unit_size(bytes_size)

    user_data.append({
        "user_id": user.id,
        "user_fullname": user.first_name + " " + user.last_name,
        "searches": searches.count(),
        "purchases": purchases.count(),
        "in_process": in_process.count(),
        "size": size_repsat,
    })

    for search in Search.objects.values('search_name').filter(user_id=user.id).annotate(
            count=Count('search_name')).order_by('-count')[:6]:
        user_searches.append({
            "search_name": search['search_name'],
            "count": search['count'],
        })

    progress = 0
    productList = []
    size = 0
    product_list = Purchase.objects.values('id', 'productList', 'aggreg_date', 'search__clouds', 'search__startDate',
                                           'search__endDate', 'search__process_id__name', 'search__search_name').filter(
        user_id=user.id, purchased=1).order_by('-aggreg_date')

    for producto in product_list:
        log_info = PATH_USERS + user.email + "/" + str(producto['id']) + "/L2ASCL_AreaProcessing.log"

        #####################################################
        #                progress bar                       #
        #####################################################
        if os.stat(log_info).st_size == 0:
            progress = 0
        else:
            if 'Linking Products...' in open(log_info).read():
                progress = 20
            if 'Merging Images...' in open(log_info).read():
                progress = 40
            if 'Deleting Images...' in open(log_info).read():
                progress = 50
            if 'Extracting SCL information...' in open(log_info).read():
                progress = 60
            if 'Finalizing...' in open(log_info).read():
                progress = 100

        with open(log_info, 'r') as myfile:
            log = myfile.read().replace('\n', '')

        for p in json.loads(producto['productList']):
            size = size + float(p['info']['size'][:-3])
            if size > 1000:
                size2 = size / 1000
                final_size = str(round(size2, 2)) + " GB"
            else:
                final_size = str(round(size, 2)) + " MB"

        productList.append({
            "search_name": producto['search__search_name'],
            "process": producto['search__process_id__name'],
            "purchase_date": producto['aggreg_date'],
            "startDate": producto['search__startDate'],
            "endDate": producto['search__endDate'],
            "clouds": producto['search__clouds'],
            "size": final_size,
            "progress": progress,
            "log_info": log,
        })
        size = 0
        final_size = 0
        log_info = ""
        progress = 0

        # lista de meses #
        month_lst = [
            {'quantity': 0, 'date': 'January'},
            {'quantity': 0, 'date': 'February'},
            {'quantity': 0, 'date': 'March'},
            {'quantity': 0, 'date': 'April'},
            {'quantity': 0, 'date': 'May'},
            {'quantity': 0, 'date': 'June'},
            {'quantity': 0, 'date': 'July'},
            {'quantity': 0, 'date': 'August'},
            {'quantity': 0, 'date': 'September'},
            {'quantity': 0, 'date': 'October'},
            {'quantity': 0, 'date': 'November'},
            {'quantity': 0, 'date': 'December'}]

    for date in Purchase.objects.filter(purchased=True, user_id=user.id).extra(
            {'month': "Extract(month from aggreg_date)"}).values_list(
            'month').annotate(Count('id')):
        date_name = calendar.month_name[int(date[0])]

        for month in month_lst:
            if month['date'] == date_name:
                month['quantity'] = date[1]

    # print(month_lst)
    return render(request, 'User_profile.html', {"user_data": user_data, "productList": productList, "dates": month_lst,
                                                 'user_searches': user_searches})


# ----------------------------------------------------------------------
def User_id(request, user_id):
    user_data = []
    user_searches = []
    user = User.objects.get(pk=user_id)

    searches = Search.objects.filter(user_id=user.id)

    purchases = Purchase.objects.filter(user_id=user.id)
    in_process = Purchase.objects.filter(user_id=user.id).exclude(progress=100)

    bytes_size = 0
    for path, directories, files in os.walk(USERS_PATH + "repsat_test_dev/"+user.email):
        for filename in files:
            bytes_size += os.lstat(os.path.join(path, filename)).st_size
        for directory in directories:
            bytes_size += os.lstat(os.path.join(path, directory)).st_size
    bytes_size += os.path.getsize(USERS_PATH + "repsat_test_dev/"+user.email)
    size_repsat = best_unit_size(bytes_size)

    user_data.append({
        "user_id": user.id,
        "user_fullname": user.first_name + " " + user.last_name,
        "searches": searches.count(),
        "purchases": purchases.count(),
        "in_process": in_process.count(),
        "size": size_repsat,
    })

    for search in Search.objects.values('search_name').filter(user_id=user.id).annotate(
            count=Count('search_name')).order_by('-count')[:6]:
        user_searches.append({
            "search_name": search['search_name'],
            "count": search['count'],
        })

    progress = 0
    productList = []
    size = 0
    product_list = Purchase.objects.values('id', 'productList', 'aggreg_date', 'search__clouds', 'search__startDate',
                                           'search__endDate', 'search__process_id__name', 'search__search_name').filter(
        user_id=user.id, purchased=1).order_by('-aggreg_date')

    for producto in product_list:
        log_info = PATH_USERS + user.email + "/" + str(producto['id']) + "/L2ASCL_AreaProcessing.log"

        #####################################################
        #                progress bar                       #
        #####################################################
        if os.stat(log_info).st_size == 0:
            progress = 0
        else:
            if 'Linking Products...' in open(log_info).read():
                progress = 20
            if 'Merging Images...' in open(log_info).read():
                progress = 40
            if 'Deleting Images...' in open(log_info).read():
                progress = 50
            if 'Extracting SCL information...' in open(log_info).read():
                progress = 60
            if 'Finalizing...' in open(log_info).read():
                progress = 100

        with open(log_info, 'r') as myfile:
            log = myfile.read().replace('\n', '')

        for p in json.loads(producto['productList']):
            size = size + float(p['info']['size'][:-3])
            if size > 1000:
                size2 = size / 1000
                final_size = str(round(size2, 2)) + " GB"
            else:
                final_size = str(round(size, 2)) + " MB"

        productList.append({
            "search_name": producto['search__search_name'],
            "process": producto['search__process_id__name'],
            "purchase_date": producto['aggreg_date'],
            "startDate": producto['search__startDate'],
            "endDate": producto['search__endDate'],
            "clouds": producto['search__clouds'],
            "size": final_size,
            "progress": progress,
            "log_info": log,
        })
        size = 0
        final_size = 0
        log_info = ""
        progress = 0

        # lista de meses #
        month_lst = [
            {'quantity': 0, 'date': 'January'},
            {'quantity': 0, 'date': 'February'},
            {'quantity': 0, 'date': 'March'},
            {'quantity': 0, 'date': 'April'},
            {'quantity': 0, 'date': 'May'},
            {'quantity': 0, 'date': 'June'},
            {'quantity': 0, 'date': 'July'},
            {'quantity': 0, 'date': 'August'},
            {'quantity': 0, 'date': 'September'},
            {'quantity': 0, 'date': 'October'},
            {'quantity': 0, 'date': 'November'},
            {'quantity': 0, 'date': 'December'}]

    for date in Purchase.objects.filter(purchased=True, user_id=user.id).extra(
            {'month': "Extract(month from aggreg_date)"}).values_list(
            'month').annotate(Count('id')):
        date_name = calendar.month_name[int(date[0])]

        for month in month_lst:
            if month['date'] == date_name:
                month['quantity'] = date[1]

    # print(month_lst)
    return render(request, 'User_profile.html', {"user_data": user_data, "productList": productList, "dates": month_lst,
                                                 'user_searches': user_searches})


# ---------------------------------------------------------
@csrf_exempt
def wsPurchasesPM(request):
    if request.is_ajax():

        table_data = []
        data = Purchase.objects.values('user_id', 'search__endDate', 'search__startDate', 'aggreg_date',
                                       'search__process_id__name', 'search__search_name').filter(
            user_id=request.POST.get('user_id'), purchased=True,
            aggreg_date__month=to_if(request.POST.get('title'))).order_by('aggreg_date')

        for producto in data:
            table_data.append({
                "search_name": producto['search__search_name'],
                "process": producto['search__process_id__name'],
                "purchase_date": str(producto['aggreg_date'])[:10],
                "startDate": str(producto['search__startDate'])[:10],
                "endDate": str(producto['search__endDate'])[:10],
            })
        return HttpResponse(json.dumps(table_data))
        # return HttpResponse({'table_data':table_data})
    else:
        raise Http404


# ----------------------------------------------------------------------
def to_if(name):
    if name == "January":
        return 1
    elif name == "February":
        return 2
    elif name == "March":
        return 3
    elif name == "April":
        return 4
    elif name == "May":
        return 5
    elif name == "June":
        return 6
    elif name == "July":
        return 7
    elif name == "August":
        return 8
    elif name == "September":
        return 9
    elif name == "October":
        return 10
    elif name == "November":
        return 11
    elif name == "December":
        return 12
    else:
        raise ValueError


# -----------------------------------------------
def best_unit_size(bytes_size):
    """
    Obtenga un tamano en bytes y conviertalo en el mejor prefijo IEC para facilitar la lectura.

    Devuelva un diccionario con tres pares de claves / valores:

    "s" - (flotador) Tamano del camino convertido a la mejor unidad para una facil lectura
    "u" - (str) El prefijo (IEC) para s (de bytes (2 ^ 0) a YiB (2 ^ 80))
    "b" - (int / long) El tamano original en bytes

    """
    for exp in range(0, 90, 10):
        bu_size = abs(bytes_size) / pow(2.0, exp)
        if int(bu_size) < 2 ** 10:
            unit = {0: "bytes", 10: "KB", 20: "MB", 30: "GB", 40: "TB", 50: "PB",
                    60: "EB", 70: "ZB", 80: "YB"}[exp]
            break
    return str(round(bu_size, 2)) + " " + unit

# ------------------------------------------------
